/*
** Copyright (C) ARM Limited, 2006-2012. All rights reserved.
*/

/* This file contains the main() program that sets the vector table location, displays a welcome message,
initializes the MPU, starts the SysTick timer, initializes the Process Stack Pointer, changes Thread mode
to Unprivileged and to use the Process Stack, then runs the main application (sorts) */


#include "scs.h"
#include "timer.h"
#include <stdio.h>
#include <math.h>
#include "defines.h"

extern unsigned int Image$$VECTORS$$Base;
extern unsigned int Image$$PROCESS_STACK$$ZI$$Limit;

extern void DUAL_TIMER_test(void);
extern void ALU_test(void);

/* Enable the FPU if required */
#ifdef __TARGET_FPU_VFP
extern void $Super$$__rt_lib_init(void);

void $Sub$$__rt_lib_init(void)
{
    /* Enable the FPU in both privileged and user modes by setting bits 20-23 to enable CP10 and CP11 coprocessors */
    SCS.CPACR = SCS.CPACR | (0xF << 20);
    $Super$$__rt_lib_init();
}
#endif


__declspec(noreturn) void main(void)
{
    /* Processor starts-up in Privileged Thread Mode using Main Stack */

    /* Named register variables */
    register unsigned int SP_PROCESS __asm("psp");
    register unsigned int CONTROL __asm("control");

    /* Display a welcome message via semihosting */
    printf("Cortex-M4 bare-metal startup example\n");

    /* Tell the processor the location of the vector table, obtained from the scatter file */
    VTOR = (unsigned int) &Image$$VECTORS$$Base;

    /* Enable Interrupt 0-1 */
    NVIC_ISER0 = 0x00000003;
    printf("NVIC_ISER0 = 0x%x\n", NVIC_ISER0);
	printf("NVIC_IABR0 = 0x%x\n\n", NVIC_IABR0);

    /* Initialize MPU */
    //SCS_init();

    /* Initialize SysTick Timer */
    //SysTick_init();

    /* Initialize Process Stack Pointer */
    SP_PROCESS = (unsigned int) &Image$$PROCESS_STACK$$ZI$$Limit;

    /* Change Thread mode to Unprivileged and to use the Process Stack */
    CONTROL = CONTROL | 3;

    /* Flush and refill pipeline with unprivileged permissions */
    __isb(0xf);

    /* Run the main application */
    //DUAL_TIMER_test();
    ALU_test();

    printf("Test finished!\n");

}
